struct Underscore{} _;

auto operator+(Underscore, Underscore){ return [](auto x, auto y){ return x+y; }; }
auto operator-(Underscore, Underscore){ return [](auto x, auto y){ return x-y; }; }

auto operator*(Underscore, Underscore){ return [](auto x, auto y){ return x*y; }; }
template<typename T> auto operator*(Underscore, T&& t){ return [y=static_cast<T&&>(t)](auto x){ return x*y; }; }
template<typename T> auto operator*(T&& t, Underscore){ return [y=static_cast<T&&>(t)](auto x){ return y*x; }; }

auto operator/(Underscore, Underscore){ return [](auto x, auto y){ return x/y; }; }
template<typename T> auto operator/(Underscore, T&& t){ return [y=static_cast<T&&>(t)](auto x){ return x/y; }; }
template<typename T> auto operator/(T&& t, Underscore){ return [y=static_cast<T&&>(t)](auto x){ return y/x; }; }

template<typename T, int n>
struct Vector
{
	T data[n];
	auto& operator[]( int k )     { return data[k]; }
	auto& operator[]( int k )const{ return data[k]; }
};

template<typename F, typename T, int n>
auto map( F&& f, Vector<T, n> const& v )
{
	Vector<T, n> res;
	for(int k=0; k<n; ++k){ res[k] = f(v[k]); }
	return res;
}

template<typename F, typename T, int n>
auto zip( F&& f, Vector<T, n> const& v, Vector<T, n> const& u )
{
	Vector<T, n> res;
	for(int k=0; k<n; ++k){ res[k] = f(v[k], u[k]); }
	return res;
}

template<typename F, typename Z, typename T, int n>
auto fold( F&& f, Z&& z, Vector<T, n> const& v )
{
	Z res = z;
	for(int k=0; k<n; ++k){ res = f(res, v[k]); }
	return res;
}

//implementations:
template<typename S, typename T, int n> auto operator*( S const& scl, Vector<T, n> const& v ){ return map( scl*_, v ); }
template<typename S, typename T, int n> auto operator*( Vector<T, n>const& v, S const& scl )      { return map( _*scl, v ); }
template<typename S, typename T, int n> auto operator/( Vector<T, n>const& v, S const& scl )      { return map( _/scl, v ); }

template<typename T, int n> auto operator+( Vector<T, n> const& v, Vector<T, n> const& u ){ return zip( _+_, v, u ); }
template<typename T, int n> auto operator-( Vector<T, n> const& v, Vector<T, n> const& u ){ return zip( _-_, v, u ); }

template<typename T, int n> auto dot( Vector<T, n> const& v, Vector<T, n> const& u ){ return fold( _+_, (T)0, zip( _*_, v, u ) ); }

template<typename T, int n> auto sqlength( Vector<T, n>&& v ){ return fold( [&](T&& t, T&& s){ return t+s*s; }, (T)0, v ); }

//-----------------------------------



template<typename T, int m, int n>//n cols, m rows
struct Matrix
{
	T data[n*m];
	auto& operator[]( int k ){ return data[k]; }
	auto& operator[]( int k )const{ return data[k]; }

	auto& operator()( int r, int c )     { return data[r*m+c]; }
	auto& operator()( int r, int c )const{ return data[r*m+c]; }

	auto col(int c)     { Vector<T, m> res; for(int i=0; i<m; ++i){ res[i] = data(i, c); } return res; }
	auto col(int c)const{ Vector<T, m> res; for(int i=0; i<m; ++i){ res[i] = data(i, c); } return res; }

	auto row(int r)     { Vector<T, n> res; for(int i=0; i<n; ++i){ res[i] = data(r, i); } return res; }
	auto row(int r)const{ Vector<T, n> res; for(int i=0; i<n; ++i){ res[i] = data(r, i); } return res; }
};

template<typename F, typename T, int m, int n>
auto map( F&& f, Matrix<T, m, n>&& mat )
{
	Matrix<T, m, n> res;
	for(int k=0; k<n*m; ++k){ res[k] = f(mat[k]); }
	return res;
}

template<typename F, typename T, int m, int n>
auto zip( F&& f, Matrix<T, m, n>&& mat1, Matrix<T, m, n>&& mat2 )
{
	Matrix<T, m, n> res;
	for(int k=0; k<n*m; ++k){ res[k] = f(mat1[k], mat2[k]); }
	return res;
}

template<typename F, typename Z, typename T, int m, int n>
auto fold( F&& f, Z&& z, Matrix<T, m, n>&& mat )
{
	Z res = z;
	for(int k=0; k<n*m; ++k){ res = f(res, mat[k]); }
	return res;
}

//implementations:
template<typename S, typename T, int m, int n> auto operator*( S const& scl, Matrix<T, m, n>const& mat ){ return map( scl*_, mat ); }
template<typename S, typename T, int m, int n> auto operator*( Matrix<T, m, n>const& mat, S const& scl ){ return map( _*scl, mat ); }
template<typename S, typename T, int m, int n> auto operator/( Matrix<T, m, n>const& mat, S const& scl ){ return map( _/scl, mat ); }

template<typename T, int m, int n> auto operator+( Matrix<T, m, n>const& mat1, Matrix<T, m, n>const& mat2 ){ return zip( _+_, mat1, mat2 ); }
template<typename T, int m, int n> auto operator-( Matrix<T, m, n>const& mat1, Matrix<T, m, n>const& mat2 ){ return zip( _-_, mat1, mat2 ); }

template<typename T, int m, int n> auto row_vectors( Matrix<T, m, n>&& mat ){ Vector<Vector<T, n>, m> res; for(int i=0; i<m; ++i){ for(int j=0; j<n; ++j){ res[i][j] = mat(i, j); } } return res; }
template<typename T, int m, int n> auto col_vectors( Matrix<T, m, n>&& mat ){ Vector<Vector<T, m>, n> res; for(int i=0; i<n; ++i){ for(int j=0; j<m; ++j){ res[i][j] = mat(j, i); } } return res; }
template<typename T, int m, int n> auto operator*( Matrix<T, m, n>&& mat, Vector<T, m>&& v ){ return map( [&](auto row){ return dot(row, v); }, row_vectors(mat) ); }

template<typename T, int n> struct PseudoVector
{
	static const int size = n;
	T data;
	int stride;
	auto& operator[]( int k )     { return data[k*stride]; }
	auto& operator[]( int k )const{ return data[k*stride]; }
};

template<typename T, int m, int n> struct MatrixRows
{
	static const int size = n;
	T const*const data;
	auto operator[]( int row )const{ return PseudoVector<T const*const, m>{ data + row * m, 1}; }
};

template<typename T, int m, int n> struct MatrixCols
{
	static const int size = m;
	T const*const data;
	auto operator[]( int col )const{ return PseudoVector<T const*const, n>{ data + col, m}; }
};


#include <type_traits>

//Generic version based on operator[] and internal static int 'size'
template<typename F, typename Indexible, int n = Indexible::size>
auto map( F&& f, Indexible const& u )
{
	using RetType = decltype( f( u[0] ) );
	Vector<RetType, n> res;
	for(int k=0; k<n; ++k){ res[k] = f(u[k]); }
	return res;
}

template<typename F, typename Indexible1, typename Indexible2, int n = Indexible1::size>
auto zip( F&& f, Indexible1 const& v, Indexible2 const& u )
{
	using RetType = decltype( f( v[0], u[0] ) );
	Vector<RetType, n> res;
	for(int k=0; k<n; ++k){ res[k] = f(v[k], u[k]); }
	return res;
}

template<typename F, typename Z, typename Indexible, int n = Indexible::size>
auto fold( F&& f, Z&& z, Indexible const& v )
{
	Z res = z;
	for(int k=0; k<n; ++k){ res = f(res, v[k]); }
	return res;
}

template<typename V, typename U> auto dot( V const& v, U const& u ){ return fold( _+_, 0, zip( _*_, v, u ) ); }

template<typename T, int n>        auto pseudo( Vector<T, n> const& v ){ return PseudoVector<T const*const, n>{ &v[0], 1 }; }
template<typename T, int m, int n> auto row_vectors2( Matrix<T, m, n> const& mat ){ return MatrixRows<T, m, n>{ &mat[0] }; }//i*n+j
template<typename T, int m, int n> auto col_vectors2( Matrix<T, m, n> const& mat ){ return MatrixCols<T, m, n>{ &mat[0] }; }//i+m*j
template<typename T, int m, int n> auto operator*( Matrix<T, m, n>const& mat, Vector<T, m>const& v ){ return map( [&](auto row){ return dot(row, v); }, row_vectors2(mat) ); }
template<typename T, int n, int m> auto operator*( Vector<T, n>const& v, Matrix<T, m, n>const& mat ){ return map( [&](auto col){ return dot(col, v); }, col_vectors2(mat) ); }

#include <iostream>
void main18()
{
	Vector<int, 2> v{ {3, 4} };
	Vector<int, 2> u{ {6, 8} };
	auto d = dot(v, u);

	Matrix<int, 2, 2> m{ {1, 2, 3, 4} };

	auto w = m * v;
	printf("%i   %i\n", w[0], w[1]);

	auto z = u * m;
	printf("%i   %i\n", z[0], z[1]);
}

template<typename S, typename T, typename F>
auto euler( S&& state, T&& step, F&& rhs ){ return zip(_+_, map(_*step, rhs(state)), state); }
